@@ -1,10 +1,10 @@
 /*
- * Remmina - The GTK+ Remote Desktop Client
+ * Remmina - The GTK Remote Desktop Client
  * Copyright (C) 2010 Jay Sorg
  * Copyright (C) 2010-2011 Vic Lee
  * Copyright (C) 2011 Marc-Andre Moreau <marcandre.moreau@gmail.com>
  * Copyright (C) 2014-2015 Antenore Gatta, Fabio Castelli, Giovanni Panozzo
- * Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+ * Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -43,14 +43,14 @@
 #include <gdk/gdkkeysyms.h>
 #include <cairo/cairo-xlib.h>
 #include <freerdp/locale/keyboard.h>
-#include <execinfo.h>
 
-static void remmina_rdp_event_on_focus_in(GtkWidget* widget, GdkEventKey* event, RemminaProtocolWidget* gp)
+static gboolean remmina_rdp_event_on_focus_in(GtkWidget* widget, GdkEventKey* event, RemminaProtocolWidget* gp)
 {
 	TRACE_CALL(__func__);
 	rfContext* rfi = GET_PLUGIN_DATA(gp);
 	rdpInput* input;
 	GdkModifierType state;
+
 #if GTK_CHECK_VERSION(3, 20, 0)
 	GdkSeat *seat;
 #else
@@ -59,7 +59,7 @@ static void remmina_rdp_event_on_focus_in(GtkWidget* widget, GdkEventKey* event,
 	GdkDevice *keyboard = NULL;
 
 	if (!rfi || !rfi->connected || rfi->is_reconnecting)
-		return;
+		return FALSE;
 
 	input = rfi->instance->input;
 	UINT32 toggle_keys_state = 0;
@@ -85,6 +85,8 @@ static void remmina_rdp_event_on_focus_in(GtkWidget* widget, GdkEventKey* event,
 
 	input->SynchronizeEvent(input, toggle_keys_state);
 	input->KeyboardEvent(input, KBD_FLAGS_RELEASE, 0x0F);
+
+	return FALSE;
 }
 
 void remmina_rdp_event_event_push(RemminaProtocolWidget* gp, const RemminaPluginRdpEvent* e)
@@ -291,13 +293,13 @@ static gboolean remmina_rdp_event_on_draw(GtkWidget* widget, cairo_t* context, R
 
 
 	if (rfi->is_reconnecting) {
-		/* freerdp is reconnecting, just show a message to the user */
+		/* FreeRDP is reconnecting, just show a message to the user */
 
 		width = gtk_widget_get_allocated_width(widget);
 		height = gtk_widget_get_allocated_height(widget);
 
 		/* Draw text */
-		msg = g_strdup_printf(_("Reconnection in progress. Attempt %d of %d…"),
+		msg = g_strdup_printf(_("Reconnection attempt %d of %d…"),
 			rfi->reconnect_nattempt, rfi->reconnect_maxattempts);
 
 		cairo_select_font_face(context, "Sans", CAIRO_FONT_SLANT_NORMAL, CAIRO_FONT_WEIGHT_BOLD);
@@ -309,7 +311,7 @@ static gboolean remmina_rdp_event_on_draw(GtkWidget* widget, cairo_t* context, R
 		g_free(msg);
 
 	}else  {
-		/* Standard drawing: we copy the surface from RDP */
+		/* Standard drawing: We copy the surface from RDP */
 
 		if (!rfi->surface)
 			return FALSE;
@@ -669,18 +671,18 @@ static gboolean remmina_rdp_event_on_key(GtkWidget* widget, GdkEventKey* event,
 			}
 		} else {
 			unicode_keyval = gdk_keyval_to_unicode(event->keyval);
-			/* Decide when whe should send a keycode or a unicode character.
-			 * - All non char keys (shift, alt, win) should be sent as keycode
+			/* Decide when whe should send a keycode or a Unicode character.
+			 * - All non char keys (Shift, Alt, Super) should be sent as keycode
 			 * - Space should be sent as keycode (see issue #1364)
-			 * - All special keys (F1-F10, numeric pad, home/end/arrows/pgup/pgdn/ins/del) keycode
-			 * - All key pressed while CTRL or ALT or WIN is down are not decoded by gdk_keyval_to_unicode(), so send it as keycode
+			 * - All special keys (F1-F10, numeric pad, Home/End/Arrows/PgUp/PgDn/Insert/Delete) keycode
+			 * - All key pressed while Ctrl or Alt or Super is down are not decoded by gdk_keyval_to_unicode(), so send it as keycode
 			 * - All keycodes not translatable to unicode chars, as keycode
-			 * - The rest as unicode char
+			 * - The rest as Unicode char
 			 */
-			if (event->keyval >= 0xfe00 ||                                                  // arrows, shift, alt, Fn, num keypad…
-				event->hardware_keycode == 0x41 ||											// space bar
-			    unicode_keyval == 0 ||                                                      // impossible to translate
-			    (event->state & (GDK_MOD1_MASK | GDK_CONTROL_MASK | GDK_SUPER_MASK)) != 0   // a modifier not recognized by gdk_keyval_to_unicode()
+			if (event->keyval >= 0xfe00 ||                                                  // Arrows, Shift, Alt, Fn, num keypad…
+				event->hardware_keycode == 0x41 ||											// Spacebar
+			    unicode_keyval == 0 ||                                                      // Impossible to translate
+			    (event->state & (GDK_MOD1_MASK | GDK_CONTROL_MASK | GDK_SUPER_MASK)) != 0   // A modifier not recognized by gdk_keyval_to_unicode()
 			    ) {
 				scancode = freerdp_keyboard_get_rdp_scancode_from_x11_keycode(event->hardware_keycode);
 				rdp_event.key_event.key_code = scancode & 0xFF;
@@ -715,11 +717,24 @@ gboolean remmina_rdp_event_on_clipboard(GtkClipboard *gtkClipboard, GdkEvent *ev
 	 * after receivina a RDP server format list in remmina_rdp_cliprdr_server_format_list()
 	 * In the latter case, we must ignore owner change */
 
+	g_debug("[RDP] owner-change event received");
+
+	rfContext *rfi = GET_PLUGIN_DATA(gp);
+	if (rfi)
+		remmina_rdp_clipboard_abort_transfer(rfi);
+
 	if (gtk_clipboard_get_owner(gtkClipboard) != (GObject*)gp) {
+		/* To do: avoid this when the new owner is another remmina protocol widget of
+		 * the same remmina application */
+		g_debug("[RDP]     new owner is different than me: new=%p me=%p. Sending local clipboard format list to server.",
+				gtk_clipboard_get_owner(gtkClipboard), (GObject*)gp);
+
 		pFormatList = remmina_rdp_cliprdr_get_client_format_list(gp);
 		rdp_event.type = REMMINA_RDP_EVENT_TYPE_CLIPBOARD_SEND_CLIENT_FORMAT_LIST;
 		rdp_event.clipboard_formatlist.pFormatList = pFormatList;
 		remmina_rdp_event_event_push(gp, &rdp_event);
+	} else {
+		g_debug("[RDP]    ... but I'm the owner!");
 	}
 	return TRUE;
 }
@@ -893,8 +908,12 @@ static void remmina_rdp_event_create_cairo_surface(rfContext* rfi)
 	int stride;
 	rdpGdi* gdi;
 
+	if (!rfi)
+		return;
+
 	gdi = ((rdpContext *)rfi)->gdi;
-	if (!rfi || !gdi)
+
+	if (!gdi)
 		return;
 
 	if (rfi->surface) {
@@ -925,10 +944,8 @@ void remmina_rdp_event_update_scale(RemminaProtocolWidget* gp)
 	if ( rfi->surface && (cairo_image_surface_get_width(rfi->surface) != gdi->width ||
 		cairo_image_surface_get_height(rfi->surface) != gdi->height) ) {
 		/* Destroys and recreate rfi->surface with new width and height */
-		if (rfi->surface) {
-			cairo_surface_destroy(rfi->surface);
-			rfi->surface = NULL;
-		}
+		cairo_surface_destroy(rfi->surface);
+		rfi->surface = NULL;
 		remmina_rdp_event_create_cairo_surface(rfi);
 	} else if ( rfi->surface == NULL ) {
 		remmina_rdp_event_create_cairo_surface(rfi);
@@ -950,7 +967,7 @@ void remmina_rdp_event_update_scale(RemminaProtocolWidget* gp)
 		/* In non scaled mode, the plugins forces dimensions of drawing area */
 		gtk_widget_set_size_request(rfi->drawing_area, width, height);
 	}
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "update-align");
+	remmina_plugin_service->protocol_plugin_update_align(gp);
 }
 
 static void remmina_rdp_event_connected(RemminaProtocolWidget* gp, RemminaPluginRdpUiObject* ui)
@@ -961,13 +978,14 @@ static void remmina_rdp_event_connected(RemminaProtocolWidget* gp, RemminaPlugin
 
 	gdi = ((rdpContext *)rfi)->gdi;
 
-	remmina_plugin_service->protocol_plugin_emit_signal(gp, "connect");
 	gtk_widget_realize(rfi->drawing_area);
 
 	remmina_rdp_event_create_cairo_surface(rfi);
 	gtk_widget_queue_draw_area(rfi->drawing_area, 0, 0, gdi->width, gdi->height);
 
 	remmina_rdp_event_update_scale(gp);
+    
+    remmina_plugin_service->protocol_plugin_signal_connection_opened(gp);
 }
 
 static void remmina_rdp_event_reconnect_progress(RemminaProtocolWidget* gp, RemminaPluginRdpUiObject* ui)
