@@ -2,7 +2,7 @@
 
 # --------------------------------------------------------------------------
 # Remmina - The GTK+ Remote Desktop Client
-# Copyright (C) 2016-2019 Antenore Gatta, Giovanni Panozzo
+# Copyright (C) 2016-2020 Antenore Gatta, Giovanni Panozzo
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -139,13 +139,12 @@ xgettext --from-code=UTF-8 \
 	--keyword=C_:1c,2 \
 	--keyword=NC_:1c,2 \
 	--keyword=g_dngettext:2,3 \
-	--add-comments \
+	--add-comments=TRANSLATORS: \
 	--files-from=po/POTFILES.in \
 	--output=po/remmina.temp.pot \
 	--package-version="$GIT_TAG" \
 	--package-name="Remmina" \
-	--copyright-holder="Antenore Gatta and Giovanni Panozzo" \
-	--msgid-bugs-address="admin@remmina.org"
+	--msgid-bugs-address="l10n@lists.remmina.org"
 
 cd "$REMMINATOP"/po || exit 1
 
